<?php
/**
 * Loulou66
 * LpsFeaturedProducts module for Prestashop
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php*
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    Loulou66.fr <contact@loulou66.fr>
 *  @copyright loulou66.fr
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */
use PrestaShop\PrestaShop\Adapter\Category\CategoryProductSearchProvider;
use PrestaShop\PrestaShop\Core\Module\WidgetInterface;
use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchContext;
use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchQuery;
use PrestaShop\PrestaShop\Core\Product\Search\SortOrder;

if (!defined('_PS_VERSION_')) {
    exit;
}

class LpsFeaturedProducts extends Module implements WidgetInterface
{
    public $templateFile;

    public function __construct()
    {
        $this->name = 'lpsfeaturedproducts';
        $this->author = 'Loulou66';
        $this->version = '8.0.1';
        $this->tab = 'front_office_features';
        $this->need_instance = 0;
        $this->ps_versions_compliancy = ['min' => '1.7.0', 'max' => _PS_VERSION_];
        $this->bootstrap = true;
        parent::__construct();
        $this->displayName = $this->l('LPS Featured products Carousel');
        $this->description = $this->l('Add a block displaying your store featured products as a CAROUSEL');
        $this->templateFile = 'module:lpsfeaturedproducts/views/templates/hook/lpsfeaturedproducts.tpl';
        $this->module_key = 'b41bb13e6e60e8b6ae2dbf79c3a1872f';
    }

    public function install()
    {
        $this->_clearCache('*');
        return parent::install()
            && Configuration::updateValue('LPSFEATUREDPRODUCTS_ENABLE', 1)
            && Configuration::updateValue('LPSFEATUREDPRODUCTS_CAT', 2)
            && Configuration::updateValue('LPSFEATUREDPRODUCTS_PROD_DISPLAY', 20)
            && Configuration::updateValue('LPSFEATUREDPRODUCTS_PROD_PER_ROW', 4)
            && Configuration::updateValue('LPSFEATUREDPRODUCTS_LOOP', 1)
            && Configuration::updateValue('LPSFEATUREDPRODUCTS_AUTOPLAY', 1)
            && Configuration::updateValue('LPSFEATUREDPRODUCTS_RANDOM', 0)
            && $this->registerHook('actionAdminControllerSetMedia')
            && $this->registerHook('actionFrontControllerSetMedia')
            && $this->registerHook('actionProductAdd')
            && $this->registerHook('actionProductUpdate')
            && $this->registerHook('actionProductDelete')
            && $this->registerHook('displayHome')
            && $this->registerHook('displayFooterProduct');
    }

    public function uninstall()
    {
        $this->_clearCache('*');
        return parent::uninstall()
            && Configuration::deleteByName('LPSFEATUREDPRODUCTS_ENABLE')
            && Configuration::deleteByName('LPSFEATUREDPRODUCTS_CAT')
            && Configuration::deleteByName('LPSFEATUREDPRODUCTS_PROD_DISPLAY')
            && Configuration::deleteByName('LPSFEATUREDPRODUCTS_PROD_PER_ROW')
            && Configuration::deleteByName('LPSFEATUREDPRODUCTS_LOOP')
            && Configuration::deleteByName('LPSFEATUREDPRODUCTS_AUTOPLAY')
            && Configuration::deleteByName('LPSFEATUREDPRODUCTS_RANDOM');
    }

    public function isUsingNewTranslationSystem()
    {
        return false;
    }

    public function hookActionProductAdd($params)
    {
        $this->_clearCache('*');
    }

    public function hookActionProductUpdate($params)
    {
        $this->_clearCache('*');
    }

    public function hookActionProductDelete($params)
    {
        $this->_clearCache('*');
    }

    public function _clearCache($template, $cache_id = null, $compile_id = null)
    {
        return parent::_clearCache($this->templateFile);
    }

    public function hookActionAdminControllerSetMedia($params)
    {
        if ($this->context->controller->controller_name == 'AdminModules'
            && Tools::getValue('configure') == $this->name
        ) {
            $this->context->controller->addCSS($this->_path . 'views/css/' . $this->name . 'admin.css');
            $this->context->controller->addJS($this->_path . 'views/js/' . $this->name . 'admin.js');
        }
    }

    public function getContent()
    {
        $html = '';
        if (Tools::isSubmit('submitlpsfeaturedproducts')) {
            if (!Validate::isInt((int) Tools::getValue('LPSFEATUREDPRODUCTS_PROD_DISPLAY'))
                || (int) Tools::getValue('LPSFEATUREDPRODUCTS_PROD_DISPLAY') <= 0
            ) {
                $html .= $this->displayError(
                    $this->l('The « Products to display » field is invalid. Please enter a positive number.')
                );
            } else {
                Configuration::updateValue(
                    'LPSFEATUREDPRODUCTS_ENABLE',
                    (int) Tools::getValue('LPSFEATUREDPRODUCTS_ENABLE')
                );
                if (!Tools::getValue('LPSFEATUREDPRODUCTS_CAT')) {
                    Configuration::updateValue('LPSFEATUREDPRODUCTS_CAT', 2);
                } else {
                    Configuration::updateValue(
                        'LPSFEATUREDPRODUCTS_CAT',
                        (int) Tools::getValue('LPSFEATUREDPRODUCTS_CAT')
                    );
                }
                Configuration::updateValue(
                    'LPSFEATUREDPRODUCTS_ENABLE',
                    (int) Tools::getValue('LPSFEATUREDPRODUCTS_ENABLE')
                );
                Configuration::updateValue(
                    'LPSFEATUREDPRODUCTS_PROD_DISPLAY',
                    (int) Tools::getValue('LPSFEATUREDPRODUCTS_PROD_DISPLAY')
                );
                Configuration::updateValue(
                    'LPSFEATUREDPRODUCTS_PROD_PER_ROW',
                    (int) Tools::getValue('LPSFEATUREDPRODUCTS_PROD_PER_ROW')
                );
                Configuration::updateValue(
                    'LPSFEATUREDPRODUCTS_LOOP',
                    (int) Tools::getValue('LPSFEATUREDPRODUCTS_LOOP')
                );
                Configuration::updateValue(
                    'LPSFEATUREDPRODUCTS_AUTOPLAY',
                    (int) Tools::getValue('LPSFEATUREDPRODUCTS_AUTOPLAY')
                );
                Configuration::updateValue(
                    'LPSFEATUREDPRODUCTS_RANDOM',
                    (int) Tools::getValue('LPSFEATUREDPRODUCTS_RANDOM')
                );
                $this->_clearCache('*');
                $html .= $this->displayConfirmation($this->l('Settings updated'));
            }
        }
        return $html . $this->renderForm();
    }

    public function renderForm()
    {
        $fields_form = [
            'form' => [
                'legend' => [
                    'title' => $this->l('Settings'),
                    'icon' => 'icon-cogs',
                ],
                'input' => [
                    [
                        'type' => 'switch',
                        'label' => $this->l('Enable carousel'),
                        'name' => 'LPSFEATUREDPRODUCTS_ENABLE',
                        'is_bool' => true,
                        'values' => [
                           [
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'categories',
                        'label' => $this->l('Category'),
                        'tree' => [
                          'id' => 'home_featured_category',
                          'selected_categories' => [Configuration::get('LPSFEATUREDPRODUCTS_CAT')],
                        ],
                        'name' => 'LPSFEATUREDPRODUCTS_CAT',
                        'desc' => $this->l('Select the category of products to display'),
                        'hint' => $this->l('Select the category of products to display'),
                        'form_group_class' => 'LPSFEATUREDPRODUCTS_ENABLE',
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Products to display'),
                        'name' => 'LPSFEATUREDPRODUCTS_PROD_DISPLAY',
                        'desc' => $this->l('Determine the total number of products to display in the carousel'),
                        'hint' => $this->l('Determine the total number of products to display in the carousel'),
                        'form_group_class' => 'LPSFEATUREDPRODUCTS_ENABLE',
                    ],
                    [
                        'type' => 'html',
                        'label' => $this->l('Products visible'),
                        'html_content' => $this->displayProdPerRow(),
                        'name' => '',
                        'desc' => $this->l('Define the number of products visible'),
                        'hint' => $this->l('Define the number of products visible'),
                        'form_group_class' => 'LPSFEATUREDPRODUCTS_ENABLE',
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Loop'),
                        'name' => 'LPSFEATUREDPRODUCTS_LOOP',
                        'is_bool' => true,
                        'form_group_class' => 'LPSFEATUREDPRODUCTS_ENABLE',
                        'values' => [
                           [
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Autoplay'),
                        'name' => 'LPSFEATUREDPRODUCTS_AUTOPLAY',
                        'is_bool' => true,
                        'form_group_class' => 'LPSFEATUREDPRODUCTS_ENABLE',
                        'values' => [
                           [
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Random display'),
                        'name' => 'LPSFEATUREDPRODUCTS_RANDOM',
                        'hint' => $this->l('Enable if you wish the products to be displayed randomly'),
                        'desc' => $this->l('Enable if you wish the products to be displayed randomly'),
                        'is_bool' => true,
                        'form_group_class' => 'LPSFEATUREDPRODUCTS_ENABLE',
                        'values' => [
                           [
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                    ],
                ],
                'submit' => [
                    'title' => $this->l('Save'),
                ],
            ],
        ];
        $lang = new Language((int) Configuration::get('PS_LANG_DEFAULT'));
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitlpsfeaturedproducts';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) .
            '&configure=' . $this->name .
            '&tab_module=' . $this->tab .
            '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = [
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        ];
        return $helper->generateForm([$fields_form]);
    }

    public function getConfigFieldsValues()
    {
        return [
            'LPSFEATUREDPRODUCTS_ENABLE' => (int) Tools::getValue(
                'LPSFEATUREDPRODUCTS_ENABLE',
                Configuration::get('LPSFEATUREDPRODUCTS_ENABLE'),
            ),
            'LPSFEATUREDPRODUCTS_CAT' => (int) Tools::getValue(
                'LPSFEATUREDPRODUCTS_CAT',
                Configuration::get('LPSFEATUREDPRODUCTS_CAT'),
            ),
            'LPSFEATUREDPRODUCTS_PROD_DISPLAY' => (int) Tools::getValue(
                'LPSFEATUREDPRODUCTS_PROD_DISPLAY',
                Configuration::get('LPSFEATUREDPRODUCTS_PROD_DISPLAY'),
            ),
            'LPSFEATUREDPRODUCTS_LOOP' => (int) Tools::getValue(
                'LPSFEATUREDPRODUCTS_LOOP',
                Configuration::get('LPSFEATUREDPRODUCTS_LOOP'),
            ),
            'LPSFEATUREDPRODUCTS_AUTOPLAY' => (int) Tools::getValue(
                'LPSFEATUREDPRODUCTS_AUTOPLAY',
                Configuration::get('LPSFEATUREDPRODUCTS_AUTOPLAY'),
            ),
            'LPSFEATUREDPRODUCTS_RANDOM' => (int) Tools::getValue(
                'LPSFEATUREDPRODUCTS_RANDOM',
                Configuration::get('LPSFEATUREDPRODUCTS_RANDOM'),
            ),
        ];
    }

    public function displayProdPerRow()
    {
        $tpl = _PS_MODULE_DIR_ . $this->name . '/views/templates/admin/lpsfpprodperrow.tpl';
        $lpsfpprodperrow = $this->context->smarty->createTemplate($tpl);
        $lpsfpprodperrow->assign(
            [
                'lpsfp_prodperrow' => (int) Tools::getValue(
                    'LPSFEATUREDPRODUCTS_PROD_PER_ROW',
                    Configuration::get('LPSFEATUREDPRODUCTS_PROD_PER_ROW')
                ),
            ]
        );
        return $lpsfpprodperrow->fetch();
    }

    public function hookActionFrontControllerSetMedia()
    {
        if (!$this->active || !Configuration::get('LPSFEATUREDPRODUCTS_ENABLE')) {
            return false;
        }
        Media::addJsDef(
            [
                'lpsfp_productsPerRow' => Configuration::get('LPSFEATUREDPRODUCTS_PROD_PER_ROW'),
                'lpsfp_loop' => (Configuration::get('LPSFEATUREDPRODUCTS_LOOP') ? true : false),
                'lpsfp_autoplay' => (Configuration::get('LPSFEATUREDPRODUCTS_AUTOPLAY') ? true : false),
            ]
        );
        $this->context->controller->registerStylesheet(
            'modules-lpsfeaturedproductsfrontcss',
            'modules/' . $this->name . '/views/css/lpsfeaturedproductsfront.css'
        );
        $this->context->controller->registerStylesheet(
            'modules-owlcarouselcss',
            'modules/' . $this->name . '/libs/owlcarousel/css/owl.carousel.css'
        );
        $this->context->controller->registerStylesheet(
            'modules-owlcarouselthemecss',
            'modules/' . $this->name . '/libs/owlcarousel/css/owl.theme.default.css'
        );
        $this->context->controller->registerJavascript(
            'modules-owlcarouseljs',
            'modules/' . $this->name . '/libs/owlcarousel/js/owl.carousel.min.js'
        );
        $this->context->controller->registerJavascript(
            'modules-lpsfeaturedproductsfrontjs',
            'modules/' . $this->name . '/views/js/lpsfeaturedproducts.js'
        );
    }

    public function renderWidget($hookName, array $configuration)
    {
        if (!$this->active || !Configuration::get('LPSFEATUREDPRODUCTS_ENABLE')) {
            return false;
        }
        $lpsfeaturedproductsrandom = Configuration::get('LPSFEATUREDPRODUCTS_RANDOM');
        if (!$lpsfeaturedproductsrandom) {
            if (!$this->isCached($this->templateFile, $this->getCacheId('lpsfeaturedproducts'))) {
                $variables = $this->getWidgetVariables($hookName, $configuration);
                if (empty($variables)) {
                    return false;
                }
                $this->context->smarty->assign($variables);
            }
            return $this->fetch($this->templateFile, $this->getCacheId('lpsfeaturedproducts'));
        } else {
            $variables = $this->getWidgetVariables($hookName, $configuration);
            if (empty($variables)) {
                return false;
            }
            $this->context->smarty->assign($variables);
            return $this->fetch($this->templateFile);
        }
    }

    public function getWidgetVariables($hookName, array $configuration)
    {
        $lpsfeaturedproducts = $this->getCatProducts();
        if (count($lpsfeaturedproducts) <= Configuration::get('LPSFEATUREDPRODUCTS_PROD_PER_ROW')) {
            $this->context->smarty->assign(['lpsfp_hidenav' => true]);
        }
        if (!empty($lpsfeaturedproducts)) {
            return ['lpsfeaturedproducts' => $lpsfeaturedproducts];
        }
        return false;
    }
    protected function getCatProducts()
    {
        $lpsfeaturedproducts_cat = new Category((int) Configuration::get('LPSFEATUREDPRODUCTS_CAT'));
        $searchProvider = new CategoryProductSearchProvider(
            $this->context->getTranslator(),
            $lpsfeaturedproducts_cat
        );
        $context = new ProductSearchContext($this->context);
        $query = new ProductSearchQuery();
        $nProducts = Configuration::get('LPSFEATUREDPRODUCTS_PROD_DISPLAY');
        if ($nProducts < 0) {
            $nProducts = 20;
        }
        $query->setResultsPerPage($nProducts)->setPage(1);
        if (Configuration::get('LPSFEATUREDPRODUCTS_RANDOM')) {
            $query->setSortOrder(SortOrder::random());
        } else {
            $query->setSortOrder(new SortOrder('product', 'position', 'asc'));
        }
        $lpsfeaturedproducts_resuslt = $searchProvider->runQuery($context, $query);
        $assembler = new ProductAssembler($this->context);
        $presenterFactory = new ProductPresenterFactory($this->context);
        $presentationSettings = $presenterFactory->getPresentationSettings();
        $presenter = $presenterFactory->getPresenter();
        $products_for_template = [];
        $lpsfeaturedproducts = $lpsfeaturedproducts_resuslt->getProducts();
        if (Configuration::get('LPSSPECIALS_RANDOM')) {
            shuffle($lpsfeaturedproducts);
        }
        foreach ($lpsfeaturedproducts as $rawProduct) {
            $products_for_template[] = $presenter->present(
                $presentationSettings,
                $assembler->assembleProduct($rawProduct),
                $this->context->language
            );
        }
        return $products_for_template;
    }
}
